const { pool } = require("../config/database/database.config");

/*
Class responsible for making queries to the 
chapterLookup database.
*/

/*
Function that adds chapter to database if it doesn't aready exist

chapter must be a string
*/
async function addChapterIfNotExists(chapter) {
  console.log("adding chapter: ", chapter);

  // don't add chapter if it exists
  const existingChapter = await getChapter(chapter);
  if (existingChapter) {
    console.log("Chapter already exists: ", chapter);
    return;
  }

  // insert SQL query
  const query = `INSERT INTO chapterLookup (name) VALUES (?)`;

  // attempt to insert chapter into table
  try {
    await pool.query(query, [chapter]);
    console.log(`Successfully added chapter: ${chapter}`);
  } catch (error) {
    console.error(`Error adding chapter: ${chapter}`, error);
    throw error;
  }
}

/*
Function to get chapter, used to check if it exists in the database or not
*/
async function getChapter(name) {
  // SQL queery to get a certain chapter
  const query = "SELECT * FROM chapterLookup WHERE name = ?";

  try {
    console.log("Getting chapter: ", name);
    const [rows] = await pool.query(query, [name]);
    if (rows.length > 0) {
      console.log("Found chapter: ", name);
      return rows[0];
    } else {
      console.log("Chapter not found: ", name);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching chapter with name ${name}:`, error);
  }
}

/*
Get the chapter ID based on the chapter name
*/
async function getChapterId(name) {
  // SQL query to get a certain chapter id
  const query = "SELECT chapterId FROM chapterLookup WHERE name = ?";

  try {
    console.log("Getting chapter id for: ", name);
    const [rows] = await pool.query(query, [name]);
    if (rows.length > 0) {
      console.log("Found chapter id: ", rows[0]);
      return rows[0].chapterId;
    } else {
      console.log("Chapter not id found: ", name);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching chapter with name ${name}:`, error);
  }
}

/*
Get the chapter ID based on the chapter name
*/
async function getChapterById(chapterId) {
  // SQL query to get a certain chapter id
  const query = "SELECT * FROM chapterLookup WHERE chapterId = ?";

  try {
    console.log("Getting chapter info for chapter id: ", chapterId);
    const [rows] = await pool.query(query, [chapterId]);
    if (rows.length > 0) {
      console.log("Found chapter with name: ", rows[0].name);

      return rows[0].name;
    } else {
      console.log("Chapter not id found: ", chapterId);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching chapter with ID ${chapterId}:`, error);
  }
}

/*
Checks if table already exists
*/
async function checkChapterTableExistence() {
  try {
    const [rows] = await pool.query("SHOW TABLES LIKE 'chapterLookup'");
    if (rows.length > 0) {
      console.log("Table `chapterLookup` exists");
    } else {
      console.log("Table `chapterLookup` does not exist");
    }
  } catch (error) {
    console.error("Error checking table existence:", error);
  }
}

module.exports = {
  checkChapterTableExistence,
  addChapterIfNotExists,
  getChapter,
  getChapterId,
  getChapterById,
};
